Stan Math Library: stan/math/prim/scal/prob/normal_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

normal_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_NORMAL_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_NORMAL_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/value_of.hpp>
13 #include <stan/math/prim/scal/meta/include_summand.hpp>
14 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
15 #include <stan/math/prim/scal/meta/max_size.hpp>
16 #include <boost/random/normal_distribution.hpp>
17 #include <boost/random/variate_generator.hpp>
18 #include <cmath>
19
20 namespace stan {
21
22 namespace math {
23
42 template <bool propto,
43 typename T_y, typename T_loc, typename T_scale>
44 typename return_type<T_y, T_loc, T_scale>::type
45 normal_log(const T_y& y, const T_loc& mu, const T_scale& sigma) {
46 static const char* function("stan::math::normal_log");
47 typedef typename stan::partials_return_type<T_y, T_loc, T_scale>::type
48 T_partials_return;
49
50 using std::log;
51 using stan::is_constant_struct;
52 using stan::math::check_positive;
53 using stan::math::check_finite;
54 using stan::math::check_not_nan;
55 using stan::math::check_consistent_sizes;
56 using stan::math::value_of;
57 using stan::math::include_summand;
58 using std::log;
59
60 // check if any vectors are zero length
61 if (!(stan::length(y)
62 && stan::length(mu)
63 && stan::length(sigma)))
64 return 0.0;
65
66 // set up return value accumulator
67 T_partials_return logp(0.0);
68
69 // validate args (here done over var, which should be OK)
70 check_not_nan(function, "Random variable", y);
71 check_finite(function, "Location parameter", mu);
72 check_positive(function, "Scale parameter", sigma);
73 check_consistent_sizes(function,
74 "Random variable", y,
75 "Location parameter", mu,
76 "Scale parameter", sigma);
77 // check if no variables are involved and prop-to
78 if (!include_summand<propto, T_y, T_loc, T_scale>::value)
79 return 0.0;
80
81 // set up template expressions wrapping scalars into vector views
82 OperandsAndPartials<T_y, T_loc, T_scale>
83 operands_and_partials(y, mu, sigma);
84
85 VectorView<const T_y> y_vec(y);
86 VectorView<const T_loc> mu_vec(mu);
87 VectorView<const T_scale> sigma_vec(sigma);
88 size_t N = max_size(y, mu, sigma);
89
90 VectorBuilder<true, T_partials_return, T_scale> inv_sigma(length(sigma));
91 VectorBuilder<include_summand<propto, T_scale>::value,
92 T_partials_return, T_scale> log_sigma(length(sigma));
93 for (size_t i = 0; i < length(sigma); i++) {
94 inv_sigma[i] = 1.0 / value_of(sigma_vec[i]);
95 if (include_summand<propto, T_scale>::value)
96 log_sigma[i] = log(value_of(sigma_vec[i]));
97 }
98
99 for (size_t n = 0; n < N; n++) {
100 // pull out values of arguments
101 const T_partials_return y_dbl = value_of(y_vec[n]);
102 const T_partials_return mu_dbl = value_of(mu_vec[n]);
103
104 // reusable subexpression values
105 const T_partials_return y_minus_mu_over_sigma
106 = (y_dbl - mu_dbl) * inv_sigma[n];
107 const T_partials_return y_minus_mu_over_sigma_squared
108 = y_minus_mu_over_sigma * y_minus_mu_over_sigma;
109
110 static double NEGATIVE_HALF = - 0.5;
111
112 // log probability
113 if (include_summand<propto>::value)
114 logp += NEG_LOG_SQRT_TWO_PI;
115 if (include_summand<propto, T_scale>::value)
116 logp -= log_sigma[n];
117 if (include_summand<propto, T_y, T_loc, T_scale>::value)
118 logp += NEGATIVE_HALF * y_minus_mu_over_sigma_squared;
119
120 // gradients
121 T_partials_return scaled_diff = inv_sigma[n] * y_minus_mu_over_sigma;
122 if (!is_constant_struct<T_y>::value)
123 operands_and_partials.d_x1[n] -= scaled_diff;
124 if (!is_constant_struct<T_loc>::value)
125 operands_and_partials.d_x2[n] += scaled_diff;
126 if (!is_constant_struct<T_scale>::value)
127 operands_and_partials.d_x3[n]
128 += -inv_sigma[n] + inv_sigma[n] * y_minus_mu_over_sigma_squared;
129 }
130
131
132 return operands_and_partials.value(logp);
133 }
134
135 template <typename T_y, typename T_loc, typename T_scale>
136 inline
137 typename return_type<T_y, T_loc, T_scale>::type
138 normal_log(const T_y& y, const T_loc& mu, const T_scale& sigma) {
139 return normal_log<false>(y, mu, sigma);
140 }
141 }
142 }
143 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::math::check_positive
bool check_positive(const char *function, const char *name, const T_y &y)
Return true if y is positive.
Definition: check_positive.hpp:68
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::NEG_LOG_SQRT_TWO_PI
const double NEG_LOG_SQRT_TWO_PI
Definition: constants.hpp:184
stan::math::normal_log
return_type< T_y, T_loc, T_scale >::type normal_log(const T_y &y, const T_loc &mu, const T_scale &sigma)
The log of the normal density for the specified scalar(s) given the specified mean(s) and deviation(s...
Definition: normal_log.hpp:45
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
max_size.hpp

[ Stan Home Page ] © 2011–2016, Stan Development Team.